﻿(function () {
    "use strict";

    var ui = WinJS.UI;
    var utils = WinJS.Utilities;


    /* STYLE BEGIN */
    var flipView = null;
    function countRetrieved(count) {

        // Step 1) Create an element to host the sub-elements in the context
        //         control.
        var contextControl = document.createElement("div");
        contextControl.className = "contextControl";

        // Step 2)  Create the event handlers for the radio buttons.  It's
        //          purpose is to handle click and translate that into
        //          flipping the FlipView to the correct page.
        var isFlipping = false;
        function radioButtonClicked(eventObject) {
            if (isFlipping) {

                // Need to set whats check back since we are mid flip.
                var page = flipView.currentPage;
                radioButtons[page].checked = true;

            } else {

                // Set the new page since we are not already flipping.
                var targetPage = eventObject.target.getAttribute("value");
                flipView.currentPage = parseInt(targetPage);

            }
        }

        // Step 3) Detect when the FlipView startsflipping to prevent
        //         subsequent clicks on the contextControl until flipping has
        //         stopped.
        flipView.addEventListener("pagevisibilitychanged", function (eventObject) {
            if (eventObject.detail.visible === true) {
                isFlipping = true;
            }
        }, false);

        // Step 4)  Create radio buttons for each page in the FlipView and
        //          add the event handler from step 3.
        var radioButtons = [];
        for (var i = 0; i < count; ++i) {

            // Create a Radio Button
            var radioButton = document.createElement("input");
            radioButton.setAttribute("type", "radio");
            radioButton.className = "flipViewRadio";
            // Set the name of the radio button group.
            radioButton.setAttribute("name", "flipperContextGroup");

            // Set the value for each button to the number of the page.
            radioButton.setAttribute("value", i);

            // Set the accessibility label
            radioButton.setAttribute("aria-label", (i + 1) + " of " + count);

            // Add the handler
            radioButton.addEventListener("click", radioButtonClicked, false);

            // Add the buttons to our radioButtons collection
            radioButtons.push(radioButton);

            // Add the radio buttons to the main container for the context control.
            contextControl.appendChild(radioButton);
        }


        // Step 5)  Set the currently checked item to the item the FlipView is
        //          currently on.
        if (count > 0) {
            radioButtons[flipView.currentPage].checked = true;
        }

        // Step 6)  The context control needs to listen to FlipView events so
        //          that it can display what is currently in view.
        flipView.addEventListener("pageselected", function () {

            // Now that flipping is done, clear the flag.
            isFlipping = false;

            // Need to set the current page.
            var page = flipView.currentPage;
            radioButtons[page].checked = true;
        }, false);

        // Step 7)  Finally, we need to add the control into the DOM.
        var contextContainer = document.querySelector(".flipViewContainer");
        contextContainer.appendChild(contextControl);
    }

    /* STYLE END */ 


    ui.Pages.define("/html/itemDetailPage.html", {
        // This function is called whenever a user navigates to this page. It
        // populates the page elements with the app's data.
        ready: function (element, options) {
            // DEMO::Animation 
            WinJS.UI.Animation.enterPage(document.querySelector('.itemdetailpage.fragment'), { top: '0px', left: '200px' });
            var appbar = document.querySelector("#appbar").winControl;            
            appbar.showOnlyCommands(["brag", "pin", "remind", "home"]);

            var item = options && options.item ? options.item : data.items.getAt(0);
            tiles.currentItem = item; // Store this in case the user pins the recipe

            /* STYLE CHANGED BEGIN */ 
            element.querySelector(".titlearea .pagetitle").textContent = item.title;
            /* STYLE CHANGED END */

            element.querySelector("article .item-title").textContent = item.title;
            element.querySelector("article .item-subtitle").textContent = item.preptime;
            /* STYLE BEGIN */

            element.querySelector(".flipViewContainer .item-image").alt = item.subtitle;
            if (item.extraImages) {
                var container = element.querySelector('.flipViewContainer');
                container.querySelector("article .item-image").style.display = "none";
                var flipViewDiv = document.createElement("div");
                flipViewDiv.className = "flipViewCustomNav";
                flipViewDiv.style.width = "400px";
                flipViewDiv.style.height = "300px";
                flipView = new WinJS.UI.FlipView(flipViewDiv);
                flipView.itemDataSource = new WinJS.Binding.List(item.extraImages).dataSource;
                flipView.itemTemplate = element.querySelector(".flipViewTemplate");
                container.appendChild(flipViewDiv);

                //  WinJS.Binding.processAll(flipView).then(function () {
                window.setTimeout(function () {
                    flipView.count().then(countRetrieved);
                }, 500);

            }
            else {
                element.querySelector(".flipViewContainer .item-image").src = item.backgroundImage;
            }

            /* STYLE END */

            /* STYLE REMOVED BEGIN */ 
            //element.querySelector("article .item-image").src = item.backgroundImage;
            //element.querySelector("article .item-image").alt = item.shortTitle;
            /* STYLE REMOVED END */


            // Display ingredients list
            var ingredients = element.querySelector("article .item-ingredients");
            for (var i = 0; i < item.ingredients.length; i++) {
                var ingredient = document.createElement("h2");
                ingredient.textContent = item.ingredients[i];
                ingredient.className = "ingredient";
                ingredients.appendChild(ingredient);
            }

            // Determine whether Italian recipes have been purchased
            var app = Windows.ApplicationModel.Store.CurrentAppSimulator;
            var license = app.licenseInformation;

            if (license.productLicenses.ItalianRecipes.isActive || item.group.key !== "Italian") {
                // Display cooking directions
                element.querySelector("article .item-directions").textContent = item.directions;
            }
            else {
                // Show the purchase price on the purchase button
                var button = document.querySelector("#purchase");

                app.loadListingInformationAsync().then(function (listing) {
                    var price = listing.productListings.ItalianRecipes.formattedPrice;
                    button.textContent = "Purchase Italian Recipes Pack for " + price;
                });

                // Handle licensechanged events
                app.licenseInformation.onlicensechanged = function () {
                    if (license.productLicenses.ItalianRecipes.isActive) {
                        // Replace purchase button with cooking directions
                        element.querySelector("article .item-directions").textContent = item.directions;
                    }
                };

                // Handle clicks of the purchase button
                button.onclick = function () {
                    app.requestProductPurchaseAsync("ItalianRecipes");
                };
            }

            var dtm = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
            //DEMO::Contracts.Share .. 
            dtm.addEventListener("datarequested", function (e) {
                var request = e.request;
                item = tiles.currentItem; 
                request.data.properties.title = item.title;

                if (media.shareFile == null) {
                    // Share recipe text
                    request.data.properties.description = "Recipe ingredients and directions";
                    var recipe = "INGREDIENTS\r\n";

                    item.ingredients.forEach(function (ingredient) {
                        recipe += ingredient + "\r\n";
                    });

                    recipe += ("\r\nDIRECTIONS\r\n" + item.directions);
                    request.data.setText(recipe);

                    // Share recipe image
                    var image = document.querySelector(".item-image");
                    var uri = image.getAttribute("src");

                    if (uri.indexOf("http://") == 0)
                        uri = new Windows.Foundation.Uri(image.src); // Remote image
                    else
                        uri = new Windows.Foundation.Uri("ms-appx:///" + image.getAttribute("src")); // Local images

                    var reference = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(uri);
                    request.data.properties.thumbnail = reference;
                    var deferral = request.getDeferral();
                    request.data.setBitmap(reference);
                    deferral.complete();
                }
                else {
                    // Share the photo or video just captured by the user
                    request.data.properties.description = "Recipe photo or video";
                    var reference = Windows.Storage.Streams.RandomAccessStreamReference.createFromFile(media.shareFile);
                    request.data.properties.thumbnail = reference;
                    request.data.setBitmap(reference);
                    media.shareFile = null;
                }
            });
        }
    })
})();
