﻿//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

/// <reference path="base-sdk.js" />
/// <reference path="Windows.Media.Capture.js" />

(function () {

    var photoFile = null;
    var videoFile = null;

    function id(elementId) {
        return document.getElementById(elementId); 
    }

    function scenario1CapturePhoto() {
        try {
            var dialog = new Windows.Media.Capture.CameraCaptureUI();
            var aspectRatio = {width: 16, height: 9};
            dialog.photoSettings.croppedAspectRatio = aspectRatio;
            dialog.captureFileAsync(Windows.Media.Capture.CameraCaptureUIMode.photo).then(function (file) {
                if (file) {
                    sdkSample.displayStatus("");
                    var scenario1BlobUrl = URL.createObjectURL(file);
                    id("scenario1ImageHolder1").src = scenario1BlobUrl;
                    photoFile = file.path;

                    // Show status
                    scenario1ActionsChangeVisibility("visible");
                } else {
                    sdkSample.displayStatus("No photo captured.");
                }
            }, function (err) {
                sdkSample.displayError(err);
            });
        } catch (err) {
            sdkSample.displayError(err);
        }
    }

    function scenario1Reset() {
        sdkSample.displayError("");
        id("scenario1ImageHolder1").src = sdkSample.placeHolderImageSource;
        photoFile = null;
        scenario1ActionsChangeVisibility("hidden");
    }

    function scenario1ActionsChangeVisibility(value) {
        id("scenario1Reset").style.visibility = value;
    }

    function scenario2CaptureVideo() {
        try {
            var dialog = new Windows.Media.Capture.CameraCaptureUI();
            dialog.videoSettings.format = Windows.Media.Capture.CameraCaptureUIVideoFormat.wmv;
            dialog.captureFileAsync(Windows.Media.Capture.CameraCaptureUIMode.video).then(function (file) {
                if (file) {
                    sdkSample.displayStatus("");
                    var scenario2Video1BlobUrl = URL.createObjectURL(file, false);
                    id("scenario2Video1").src = scenario2Video1BlobUrl;
                    videoFile = file.path;

                    // Show status
                    scenario2ActionsChangeVisibility("visible");
                } else {
                    sdkSample.displayStatus("No video captured.");
                }
            }, function (err) {
                sdkSample.displayError(err);
            });
        } catch (err) {
            sdkSample.displayError(err);
        }
    }

    function scenario2Reset() {
        sdkSample.displayError("");
        id("scenario2Video1").src = "";
        videoFile = null;
        scenario2ActionsChangeVisibility("hidden");
    }

    function scenario2ActionsChangeVisibility(value) {
        id("scenario2Reset").style.visibility = value;
    }

    function initialize() {
        // Add any initialization code here
        scenario1Reset();
        scenario2Reset();

        id("scenario1Open").addEventListener("click", /*@static_cast(EventListener)*/scenario1CapturePhoto, false);
        id("scenario1Reset").addEventListener("click", /*@static_cast(EventListener)*/scenario1Reset, false);
        id("scenario2Open").addEventListener("click", /*@static_cast(EventListener)*/scenario2CaptureVideo, false);
        id("scenario2Reset").addEventListener("click", /*@static_cast(EventListener)*/scenario2Reset, false);
    }

    function onScenarioChanged() {
        // Do any necessary clean up on the output, the scenario id
        // can be obtained from sdkSample.scenarioId.
        scenario1Reset();
        scenario2Reset();
    }

    function activatedHandler(e) {
        // Resume user's state
        var photoFilePath = WinJS.Application.sessionState.appPhotoFile;
        if (photoFilePath) {
            Windows.Storage.StorageFile.getFileFromPathAsync(photoFilePath).then(function (file) {
                photoFile = photoFilePath;
                id("scenario1ImageHolder1").src = URL.createObjectURL(file);
                scenario1ActionsChangeVisibility("visible");
            });
        }

        var videoFilePath = WinJS.Application.sessionState.appVideoFile;
        if (videoFilePath) {
            Windows.Storage.StorageFile.getFileFromPathAsync(videoFilePath).then(function (file) {
                videoFile = videoFilePath;
                id("scenario2Video1").src = URL.createObjectURL(file, false);
                scenario2ActionsChangeVisibility("visible");
            });
        }
    }

    function checkpointHandler(e) {
        // Save user's state
        WinJS.Application.sessionState.appPhotoFile = photoFile;
        WinJS.Application.sessionState.appVideoFile = videoFile;
    }

    document.addEventListener("DOMContentLoaded", initialize, false);

    WinJS.Application.addEventListener("activated", activatedHandler);
    WinJS.Application.addEventListener("checkpoint", checkpointHandler);
})();