﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage.xaml class.
//

#include "pch.h"
#include "MainPage.xaml.h"
using namespace MediaCaptureCPP;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::System;
using namespace Windows::Foundation;
using namespace Platform;
using namespace CppSamplesUtils;
using namespace Windows::UI;
using namespace Windows::UI::Core;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::Storage;
using namespace Windows::Media::MediaProperties;
using namespace Windows::Storage::Streams;

using namespace Windows::UI::Xaml::Media::Imaging;


MainPage::MainPage()
{
    InitializeComponent();

    ScenarioList->SelectionChanged += ref new SelectionChangedEventHandler(this, &MainPage::ScenarioList_SelectionChanged);
    ListBoxItem^ startingScenario = nullptr;

    auto ps = SuspensionManager::SessionState();
    if (ps->HasKey("SelectedScenario"))
    {
        PropertyValue^ pv;
        String^ item = dynamic_cast<String^>(ps->Lookup("SelectedScenario"));
        startingScenario = dynamic_cast<ListBoxItem^>(this->FindName(item));
    }

    ScenarioList->SelectedItem = startingScenario != nullptr ? startingScenario : Scenario1;

    Scenario1Init();
    Scenario2Init();
    Scenario3Init();

    _viewStateHandlerToken = ApplicationView::GetForCurrentView()->ViewStateChanged +=
    ref new TypedEventHandler<ApplicationView^,ApplicationViewStateChangedEventArgs^>(this, &MainPage::Page_ViewStateChanged);
	Application::Current->Suspending += ref new Windows::UI::Xaml::SuspendingEventHandler(this, &MainPage::Suspending);
    Application::Current->Resuming += ref new Windows::Foundation::EventHandler<Platform::Object^>(this, &MainPage::Resuming);	
}

MainPage::~MainPage()
{
}

void MainPage::Scenario1Init()
{
    btnStartDevice1->IsEnabled = true;
    btnStartPreview1->IsEnabled = false;    
    btnStartStopRecord1->IsEnabled = false;
    m_bRecording = false;
    m_bPreviewing = false;    
    btnStartStopRecord1->Content = "StartRecord";
    btnTakePhoto1->IsEnabled = false;
    previewElement1->Source = nullptr;
    playbackElement1->Source = nullptr;
    imageElement1->Source= nullptr;
    ShowStatusMessage("");
    sldBrightness->IsEnabled = false;
    sldContrast->IsEnabled = false;
    m_bSuspended = false;
}

void MainPage::Scenario1Reset()
{
    ShowStatusMessage("");
}

void MainPage::Scenario2Init()
{
    btnStartDevice2->IsEnabled = true;
    btnStartPreview2->IsEnabled = false;    
    btnStartStopRecord2->IsEnabled = false;
    m_bRecording = false;
    m_bPreviewing = false;
    m_bEffectAdded = false;
    btnStartStopRecord2->Content = "StartRecord";
    btnTakePhoto2->IsEnabled = false;
    previewElement2->Source = nullptr;
    playbackElement2->Source = nullptr;
    imageElement2->Source= nullptr;
    ShowStatusMessage("");
    chkAddRemoveEffect->IsChecked = false;
    chkAddRemoveEffect->IsEnabled = false;    
    EnumerateWebcamsAsync();    
    m_bSuspended = false;
}

void MainPage::Scenario2Reset()
{
    ShowStatusMessage("");

}

void MainPage::Scenario3Init()
{
    btnStartDevice3->IsEnabled = true;    
    btnStartStopRecord3->IsEnabled = false;
    m_bRecording = false;
    m_bPreviewing = false;
    playbackElement3->Source = nullptr;
    m_bSuspended = false;
    ShowStatusMessage("");
    
}

void MainPage::Scenario3Reset()
{
    ShowStatusMessage("");
 
}

void MainPage::ResetAll()
{
    previewElement1->Source = nullptr;
    playbackElement1->Source = nullptr;
    imageElement1->Source= nullptr;
    previewElement2->Source = nullptr;
    playbackElement2->Source = nullptr;
    imageElement2->Source= nullptr;
    playbackElement3->Source = nullptr;

    Scenario1Input->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    Scenario1Output->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    Scenario1Reset();
    Scenario2Input->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    Scenario2Output->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    Scenario2Reset();
    Scenario3Input->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    Scenario3Output->Visibility = Windows::UI::Xaml::Visibility::Collapsed;
    Scenario3Reset();
}

void MainPage::Suspending(Object^ sender,  Windows::ApplicationModel::SuspendingEventArgs^ e)
{
    try{
        if(m_bRecording)
        {
            ShowStatusMessage("Stopping Record on suspension");
            
            auto opStopRecord = m_mediaCaptureMgr->StopRecordAsync();
            m_bSuspended = true;
            opStopRecord->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnStopRecordCB);
        }
        if(m_bPreviewing)
        {
           ShowStatusMessage("Stopping Preview on suspension");
            
            auto opStopPreview = m_mediaCaptureMgr->StopPreviewAsync();
            m_bSuspended = true;
            opStopPreview->Completed = ref new AsyncActionCompletedHandler( this, &MainPage::OnStopPreviewCB);                     
        }
    }catch(Platform::Exception ^e)
    {
        ShowExceptionMessage(e);
    }    
}

void MainPage::Resuming(Object^ sender, Object^ e)
{
    ScenarioList_SelectionChanged(sender, nullptr);
}
        
void MainPage::ScenarioList_SelectionChanged(Object^ sender, SelectionChangedEventArgs^ e)
{
    ResetAll();

    ListBoxItem^ selectedListBoxItem = dynamic_cast<ListBoxItem^>(ScenarioList->SelectedItem);
    SuspensionManager::SessionState()->Insert("SelectedScenario", selectedListBoxItem->Name);

    if (ScenarioList->SelectedIndex == 0)
    {
        Scenario1Input->Visibility = Windows::UI::Xaml::Visibility::Visible;
        Scenario1Output->Visibility = Windows::UI::Xaml::Visibility::Visible;
        Scenario1Init();
    }
    else if (ScenarioList->SelectedIndex == 1)
    {
        Scenario2Input->Visibility = Windows::UI::Xaml::Visibility::Visible;
        Scenario2Output->Visibility = Windows::UI::Xaml::Visibility::Visible;
        Scenario2Init();
    }
    else if (ScenarioList->SelectedIndex == 2)
    {
        Scenario3Input->Visibility = Windows::UI::Xaml::Visibility::Visible;
        Scenario3Output->Visibility = Windows::UI::Xaml::Visibility::Visible;
        Scenario3Init();
    }
}


void MainPage::Footer_Click(Object^ sender, RoutedEventArgs^ e)
{
    auto _hyperlinkButton = (HyperlinkButton^)sender;
    auto uri = ref new Windows::Foundation::Uri((String^)_hyperlinkButton->Tag);

    auto launchUriOperation = Windows::System::Launcher::LaunchUriAsync(uri);
    launchUriOperation->Completed = ref new AsyncOperationCompletedHandler<bool>([](IAsyncOperation<bool>^ operation, AsyncStatus /*status*/ ) {});
}

void MainPage::Page_ViewStateChanged(ApplicationView^ sender, ApplicationViewStateChangedEventArgs^ e)
{
    switch(e->ViewState)
    {
    case ApplicationViewState::Filled:
        VisualStateManager::GoToState(this, "Filled", false);
        break;
    case ApplicationViewState::FullScreenLandscape:
        VisualStateManager::GoToState(this, "FullScreenLandscape", false);
        break;
    case ApplicationViewState::Snapped:
        VisualStateManager::GoToState(this, "Snapped", false);
        break;
    case ApplicationViewState::FullScreenPortrait:
        VisualStateManager::GoToState(this, "FullScreenPortrait", false);
        break;
    }
}

void MainPage::lstEnumedDevices_SelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ e)
{
    btnStartDevice2->IsEnabled = true;
    btnStartPreview2->IsEnabled = false;    
    btnStartStopRecord2->IsEnabled = false;
    m_bRecording = false;
    btnStartStopRecord2->Content = "StartRecord";
    btnTakePhoto2->IsEnabled = false;
    previewElement2->Source = nullptr;
    playbackElement2->Source = nullptr;
    imageElement2->Source= nullptr;
    chkAddRemoveEffect->IsEnabled = false;
    m_bEffectAdded = false;    
    ShowStatusMessage("");
}


void MainPage::EnumerateWebcamsAsync()
{   
    try
    {
        ShowStatusMessage("Enumerating Webcams...");
        m_devInfoCollection = nullptr;
        
        EnumedDeviceList2->Items->Clear();

        auto enumOp = DeviceInformation::FindAllAsync(DeviceClass::VideoCapture);                
        enumOp->Completed = ref new AsyncOperationCompletedHandler<DeviceInformationCollection^>(this, &MainPage::OnWebcamsEnumeratedCB);        
    }
    catch (Platform::Exception^ e)
    {
       ShowExceptionMessage(e);
    }
}

void MainPage::OnWebcamsEnumeratedCB(IAsyncOperation<DeviceInformationCollection^>^ asyncOp, AsyncStatus)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnWebcamsEnumeratedComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, asyncOp);
}

void MainPage::OnWebcamsEnumeratedComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try{
        IAsyncOperation<DeviceInformationCollection^>^ asyncOp = safe_cast<IAsyncOperation<DeviceInformationCollection^>^>(e->Context);

        if (asyncOp->Status == AsyncStatus::Completed)
        {
            m_devInfoCollection = asyncOp->GetResults();            
             if(m_devInfoCollection == nullptr || m_devInfoCollection->Size == 0)
            {
                ShowStatusMessage("No WebCams found.");
            }
            else
            {
                for(unsigned int i = 0; i < m_devInfoCollection->Size; i++)
                {
                    auto devInfo = m_devInfoCollection->GetAt(i);
                    EnumedDeviceList2->Items->Append(devInfo->Name);
                }                
                EnumedDeviceList2->SelectedIndex = 0;
                ShowStatusMessage("Enumerating Webcams completed successfully.");
                btnStartDevice2->IsEnabled = true; 
            }
        }    
    }
    catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);

    }
}

void MainPage::btnStartDevice_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
    try{
        EnableButton(false, "StartDevice");
        ShowStatusMessage("Starting device");
        m_mediaCaptureMgr = ref new Windows::Media::Capture::MediaCapture();
        if(ScenarioList->SelectedIndex ==0)
        {   
            IAsyncAction^ opCapture = m_mediaCaptureMgr->InitializeAsync();
            opCapture->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnStartDeviceCB);     
        }
        if(ScenarioList->SelectedIndex == 1)
        {   auto settings = ref new Windows::Media::Capture::MediaCaptureInitializationSettings();
            auto chosenDevInfo = m_devInfoCollection->GetAt(EnumedDeviceList2->SelectedIndex);
            settings->VideoDeviceId = chosenDevInfo->Id;
            IAsyncAction^ opCapture = m_mediaCaptureMgr->InitializeAsync(settings);
            opCapture->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnStartDeviceCB);
        }
        if(ScenarioList->SelectedIndex == 2)
        {   auto settings = ref new Windows::Media::Capture::MediaCaptureInitializationSettings();            
            settings->StreamingCaptureMode = Windows::Media::Capture::StreamingCaptureMode::Audio;
            IAsyncAction^ opCapture = m_mediaCaptureMgr->InitializeAsync(settings);
            opCapture->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnStartDeviceCB);
        }
    }
    catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}

void MainPage::OnStartDeviceCB(IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnStartDeviceComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}

void MainPage::OnStartDeviceComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    IAsyncAction^ op = safe_cast<IAsyncAction ^>(e->Context);
    try
    {
        EnableButton(true, "StartPreview");
        EnableButton(true, "StartStopRecord");
        EnableButton(true, "TakePhoto");
        if(op->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage("Device initialized successful");            
        }
        else
        {
            ShowStatusMessage("Device initialization error");
        }

        if(ScenarioList->SelectedIndex == 0)
        {
            // Brightness
            SetupVideoDeviceControl(m_mediaCaptureMgr->VideoDeviceController->Brightness, sldBrightness);
            // Contrast
            SetupVideoDeviceControl(m_mediaCaptureMgr->VideoDeviceController->Contrast, sldContrast);
        }
        if(ScenarioList->SelectedIndex == 1)
        {
            chkAddRemoveEffect->IsEnabled = true;
        }

    }catch(Platform::Exception ^e)
    {
        ShowExceptionMessage(e);
    }
}


void MainPage::btnStartPreview_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
    m_bPreviewing = false;
    try{
        ShowStatusMessage("Starting preview");
        EnableButton(false, "StartPreview");
        if(ScenarioList->SelectedIndex == 0)
        {            
            previewElement1->Source = m_mediaCaptureMgr;        
        }
        if(ScenarioList->SelectedIndex == 1)
        {
            previewElement2->Source = m_mediaCaptureMgr;        
        }
        IAsyncAction^ opStartPreview = m_mediaCaptureMgr->StartPreviewAsync();

        opStartPreview->Completed =ref new AsyncActionCompletedHandler(this, &MainPage::OnStartPreviewCB);
        m_bPreviewing = true;
    }
    catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}

void MainPage::OnStartPreviewCB(Windows::Foundation::IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnStartPreviewComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}

void MainPage::OnStartPreviewComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try{
        IAsyncAction^ op = safe_cast<IAsyncAction^>(e->Context);
        if(op->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage("Start preview successful");
        }
        else
        {
            m_bPreviewing = false;
            ShowStatusMessage("Start preview error");
        }

    }
    catch(Platform::Exception^ e)
    {
        m_bPreviewing = false;
        ShowExceptionMessage(e);
    }
}

void MainPage::OnStopPreviewCB(Windows::Foundation::IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnStopPreviewComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}

void MainPage::OnStopPreviewComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try{
        IAsyncAction^ op = safe_cast<IAsyncAction^>(e->Context);
        if(op->Status == AsyncStatus::Completed)
        {
            m_bPreviewing = false;
            ShowStatusMessage("Stop preview successful");
        }
        else
        {
            m_bPreviewing = false;
            ShowStatusMessage("Stop preview error");
        }

        if(ScenarioList->SelectedIndex == 0)
        {            
            previewElement1->Source = nullptr;        
        }
        if(ScenarioList->SelectedIndex == 1)
        {
            previewElement2->Source = nullptr;        
        }

    }
    catch(Platform::Exception^ e)
    {
        m_bPreviewing = false;
        ShowExceptionMessage(e);
        if(ScenarioList->SelectedIndex == 0)
        {            
            previewElement1->Source = nullptr;        
        }
        if(ScenarioList->SelectedIndex == 1)
        {
            previewElement2->Source = nullptr;        
        }

    }
}


void MainPage::btnTakePhoto_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
    try
    {
        ShowStatusMessage("Taking photo");   
        EnableButton(false, "TakePhoto");
            
        auto opCreateFile = KnownFolders::PicturesLibrary->CreateFileAsync(PHOTO_FILE_NAME);
        opCreateFile->Completed = ref new AsyncOperationCompletedHandler<StorageFile^>( this, &MainPage::OnPhotoFileCreatedCB);
    }
    catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
        EnableButton(true, "TakePhoto");
    }    
}

void MainPage::OnPhotoFileCreatedCB(IAsyncOperation<StorageFile^>^ opFile, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnPhotoFileCreatedComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, opFile);
    
}

void MainPage::OnPhotoFileCreatedComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{   

     try{                                        
            IAsyncOperation<StorageFile^>^ op = safe_cast<IAsyncOperation<StorageFile^>^>(e->Context);
            if(op->Status == AsyncStatus::Completed)
            {
                ShowStatusMessage("Create photo file successful");
                this->m_photoStorageFile = safe_cast<IAsyncOperation<StorageFile^>^>(e->Context)->GetResults();
                auto photoFile = this->m_photoStorageFile;
                ImageEncodingProperties^ imageProperties = ref new ImageEncodingProperties();
                imageProperties->Subtype = "JPEG";
                imageProperties->Width = 320;
                imageProperties->Height = 240;     
                auto opCapturePhoto = m_mediaCaptureMgr->CapturePhotoToStorageFileAsync(imageProperties, this->m_photoStorageFile);                        

                opCapturePhoto->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnTakePhotoCB);                
            }
            else
            {
                ShowStatusMessage("Create photo file error");
                EnableButton(true, "TakePhoto");
            }
        }
        catch(Platform::Exception^ e)
        {
            EnableButton(true, "TakePhoto");
            ShowExceptionMessage(e);
        }
}

void MainPage::OnTakePhotoCB(Windows::Foundation::IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
        Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnTakePhotoComplete, Platform::CallbackContext::Any);
       this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}
void MainPage::OnTakePhotoComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try
    {
        EnableButton(true, "TakePhoto");
        IAsyncAction^ op = safe_cast<IAsyncAction^>(e->Context);        
        if(op->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage("Photo taken");

            auto opOpenFile = this->m_photoStorageFile->OpenAsync(FileAccessMode::Read);
                                       
            opOpenFile->Completed = ref new AsyncOperationCompletedHandler<IRandomAccessStream^>(this, &MainPage::OnPhotoFileOpenedCB);       
        }
        else
        {
            ShowStatusMessage("Take photo unsuccessful");            
        }
    }
    catch(Platform::Exception ^ e)
    {        
        ShowExceptionMessage(e);
    }
}

void MainPage::OnPhotoFileOpenedCB(Windows::Foundation::IAsyncOperation<Windows::Storage::Streams::IRandomAccessStream^>^ photoStream, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnPhotoFileOpenedComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, photoStream);
}
void MainPage::OnPhotoFileOpenedComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try{

        IAsyncOperation<IRandomAccessStream^>^ asyncOp = safe_cast<IAsyncOperation<IRandomAccessStream^>^>(e->Context);
        if (asyncOp->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage(this->m_photoStorageFile->Path);

            auto stream = asyncOp->GetResults();
            auto bmpimg = ref new BitmapImage();

            bmpimg->SetSource(stream);
            if(ScenarioList->SelectedIndex == 0)
            {
                imageElement1->Source = bmpimg;
            }
            else if(ScenarioList->SelectedIndex == 1)
            {
                imageElement2->Source = bmpimg;
            }
        }
        else
        {
            ShowStatusMessage("Open photo file error");
        }
    }catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}


void MainPage::btnStartStopRecord_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{  
    try
    {
        String ^fileName;
        EnableButton(false, "StartStopRecord");

        if(!m_bRecording)
        {
            ShowStatusMessage("Starting Record");           
            
            if(ScenarioList->SelectedIndex == 0 || ScenarioList->SelectedIndex == 1)
            {
                fileName = VIDEO_FILE_NAME;
            }
            else
            {
                fileName= AUDIO_FILE_NAME;
            }            
            auto opCreateFile = KnownFolders::VideosLibrary->CreateFileAsync(fileName);
            opCreateFile->Completed = ref new AsyncOperationCompletedHandler<StorageFile^>( this, &MainPage::OnRecordFileCreatedCB);
            
        }
        else
        {
            ShowStatusMessage("Stopping Record");
            
            auto opStopRecord = m_mediaCaptureMgr->StopRecordAsync();
            opStopRecord->Completed = ref new AsyncActionCompletedHandler( this, &MainPage::OnStopRecordCB);
            
        }
    }
    catch(Platform::Exception^ e)
    {    
        EnableButton(true, "StartStopRecord");
        ShowExceptionMessage(e);        
    }    
}


void MainPage::OnRecordFileCreatedCB(IAsyncOperation<StorageFile^>^ opFile, AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnRecordFileCreatedComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, opFile);    
}

void MainPage::OnRecordFileCreatedComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{   
     try{   
            IAsyncOperation<StorageFile^>^ op = safe_cast<IAsyncOperation<StorageFile^> ^>(e->Context);
            if(op->Status == AsyncStatus::Completed)
            {
                ShowStatusMessage("Create record file successful");
                this->m_recordStorageFile = op->GetResults();    

                MediaEncodingProfile^ recordProfile= nullptr;
				if(ScenarioList->SelectedIndex == 0 || ScenarioList->SelectedIndex == 1)
				{
					recordProfile = MediaEncodingProfile::CreateMp4(Windows::Media::MediaProperties::VideoEncodingQuality::Auto);
				}
				else
				{
					recordProfile = MediaEncodingProfile::CreateM4a(Windows::Media::MediaProperties::AudioEncodingQuality::Auto);
				}
            
                auto opStartRecord = m_mediaCaptureMgr->StartRecordToStorageFileAsync(recordProfile, this->m_recordStorageFile);                        

                opStartRecord->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnStartRecordCB);                
            }
            else
            {
                EnableButton(true, "StartStopRecord");
                ShowStatusMessage("Create record file error");
            }            
        }
        catch(Platform::Exception^ e)
        {
            EnableButton(true, "StartStopRecord");
            ShowExceptionMessage(e);
        }
}

void MainPage::OnStartRecordCB(Windows::Foundation::IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnStartRecordComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}
void MainPage::OnStartRecordComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{    
    try
    {
        EnableButton(true, "StartStopRecord");         
        IAsyncAction^ asyncOp = safe_cast<IAsyncAction^>(e->Context);
        if (asyncOp->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage("Start Record successful");            
            m_bRecording = true;
            SwitchRecordButtonContent();                    
        }
        else
        {
            m_bRecording = false;       
            ShowStatusMessage("Start Record error");                        
        }
    }
    catch(Platform::Exception ^ e)
    {
        m_bRecording = false;             
        ShowExceptionMessage(e);        
    }
}


void MainPage::OnStopRecordCB(Windows::Foundation::IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
       Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnStopRecordComplete, Platform::CallbackContext::Any);
       this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}
void MainPage::OnStopRecordComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try
    {
        m_bRecording = false;   
        EnableButton(true, "StartStopRecord");        
        SwitchRecordButtonContent();        
                 
        IAsyncAction^ asyncOp = safe_cast<IAsyncAction^>(e->Context);
        if (asyncOp->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage("Stop record successful");
            if(!m_bSuspended)
            {
                auto opOpenFile = this->m_recordStorageFile->OpenAsync(FileAccessMode::Read);                                       
                opOpenFile->Completed = ref new AsyncOperationCompletedHandler<IRandomAccessStream^>(this, &MainPage::OnRecordFileOpenedCB);              
                
            }
        }
        else
        {
            ShowStatusMessage("Stop Record error");            
        }
    }
    catch(Platform::Exception ^ e)
    {
        ShowExceptionMessage(e);        
    }
}

void MainPage::OnRecordFileOpenedCB(Windows::Foundation::IAsyncOperation<Windows::Storage::Streams::IRandomAccessStream^>^ recordStream, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnRecordFileOpenedComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, recordStream);
}

void MainPage::OnRecordFileOpenedComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try{

        IAsyncOperation<IRandomAccessStream^>^ asyncOp = safe_cast<IAsyncOperation<IRandomAccessStream^>^>(e->Context);
        if (asyncOp->Status == AsyncStatus::Completed)
        {
            ShowStatusMessage(this->m_recordStorageFile->Path);
            auto stream = asyncOp->GetResults();                 
            if(ScenarioList->SelectedIndex == 0)
            {
                playbackElement1->AutoPlay = true;
                playbackElement1->SetSource(stream, this->m_recordStorageFile->FileType);
                playbackElement1->Play();
            }
            else if(ScenarioList->SelectedIndex == 1)
            {
                playbackElement2->AutoPlay = true;
                playbackElement2->SetSource(stream, this->m_recordStorageFile->FileType);
                playbackElement2->Play();
            }
            else if(ScenarioList->SelectedIndex == 2)
            {
                playbackElement3->AutoPlay = true;
                playbackElement3->SetSource(stream, this->m_recordStorageFile->FileType);
                playbackElement3->Play();
            }
        }
        else
        {
            ShowStatusMessage("Open record file error");
        }
    }catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}


void MainPage::chkAddRemoveEffect_Checked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
    try{
        chkAddRemoveEffect->IsEnabled = false;
        m_bEffectAdded = true;
        IAsyncAction^ op = m_mediaCaptureMgr->AddEffectAsync(Windows::Media::Capture::MediaStreamType::VideoPreview,"GrayscaleTransform.GrayscaleEffect", nullptr);
        op->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnEffectAddedOrRemovedCB);   
        
    }catch(Platform::Exception ^e)
    {
        ShowExceptionMessage(e);
    }
}

void MainPage::chkAddRemoveEffect_Unchecked(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
    try{
        chkAddRemoveEffect->IsEnabled = false;
        m_bEffectAdded = false;
        IAsyncAction^ op = m_mediaCaptureMgr->ClearEffectsAsync(Windows::Media::Capture::MediaStreamType::VideoPreview);
        op->Completed = ref new AsyncActionCompletedHandler(this, &MainPage::OnEffectAddedOrRemovedCB);   
        
    }catch(Platform::Exception ^e)
    {
        ShowExceptionMessage(e);
    }
}

void MainPage::OnEffectAddedOrRemovedCB(Windows::Foundation::IAsyncAction^ op, Windows::Foundation::AsyncStatus status)
{
    Windows::UI::Core::InvokedHandler^ handler = ref new Windows::UI::Core::InvokedHandler(this, &MainPage::OnEffectAddedorRemovedComplete, Platform::CallbackContext::Any);
    this->Dispatcher->InvokeAsync(Windows::UI::Core::CoreDispatcherPriority::High, handler, this, op);
}

void MainPage::OnEffectAddedorRemovedComplete(Platform::Object^ sender, Windows::UI::Core::InvokedHandlerArgs^ e)
{
    try{
        IAsyncAction ^op = safe_cast<IAsyncAction^>(e->Context);
        
        if(op->Status == AsyncStatus::Completed)
        {
            if(m_bEffectAdded == true)
            {
                ShowStatusMessage("Add effect successful");
            }
            else
            {
                ShowStatusMessage("Clear effect successful");
            }
        }
        else
        {
            if(m_bEffectAdded == false)
            {
                ShowStatusMessage("Add effect error");
            }
            else
            {
                ShowStatusMessage("Clear effect error");
            }
        }
        chkAddRemoveEffect->IsEnabled = true;
    }catch(Platform::Exception ^e)
    {
        chkAddRemoveEffect->IsEnabled = true;
        ShowExceptionMessage(e);
    }
}

void MainPage::SetupVideoDeviceControl(Windows::Media::Devices::MediaDeviceControl^ videoDeviceControl, Slider^ slider)
{
    try
    {
        if ((videoDeviceControl->Capabilities)->Supported)
        {
            slider->IsEnabled = true;
            slider->Maximum = videoDeviceControl->Capabilities->Max;
            slider->Minimum = videoDeviceControl->Capabilities->Min;
            slider->StepFrequency = videoDeviceControl->Capabilities->Step;
            double controlValue = 0;
            if (videoDeviceControl->TryGetValue(&controlValue))
            {
                slider->Value = controlValue;
            }
        }
        else
        {
            slider->IsEnabled = false;            
        }
    }
    catch (Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}

// VideoDeviceControllers
void MainPage::sldBrightness_ValueChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventArgs^ e)
{
    try
    {        
        bool succeeded = m_mediaCaptureMgr->VideoDeviceController->Brightness->TrySetValue(sldBrightness->Value);
        if(!succeeded)
        {
            ShowStatusMessage("Set Brightness failed");
        }
    }
    catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}

void MainPage::sldContrast_ValueChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::Primitives::RangeBaseValueChangedEventArgs  ^e)
{
    try{
        bool succeeded =  m_mediaCaptureMgr->VideoDeviceController->Contrast->TrySetValue(sldContrast->Value);
        if(!succeeded)
        {
            ShowStatusMessage("Set Contrast failed");
        }
    }
    catch(Platform::Exception^ e)
    {
        ShowExceptionMessage(e);
    }
}


void MainPage::ShowStatusMessage(Platform::String^ text)
{
    StatusMessage->Text = text;
}

void MainPage::ShowExceptionMessage(Platform::Exception^ ex)
{
    ShowStatusMessage(ex->Message);
}

void MainPage::SwitchRecordButtonContent()
{
    if(ScenarioList->SelectedIndex == 0)
    {
        if(m_bRecording)
        {
            btnStartStopRecord1->Content="StopRecord";
        }
        else
        {
            btnStartStopRecord1->Content="StartRecord";
        }
    }
    if(ScenarioList->SelectedIndex == 1)
    {
        if(m_bRecording)
        {
            btnStartStopRecord2->Content="StopRecord";
        }
        else
        {
            btnStartStopRecord2->Content="StartRecord";
        }
    }
    if(ScenarioList->SelectedIndex == 2)
    {
        if(m_bRecording)
        {
            btnStartStopRecord3->Content="StopRecord";
        }
        else
        {
            btnStartStopRecord3->Content="StartRecord";
        }
    }
}
void MainPage::EnableButton(bool enabled, String^ name)
{
    if(name->Equals("StartDevice"))
    {
        if(ScenarioList->SelectedIndex == 0)
        {
            btnStartDevice1->IsEnabled = enabled;
        }
        else if(ScenarioList->SelectedIndex == 1)
        {
            btnStartDevice2->IsEnabled = enabled;
        }
        else if(ScenarioList->SelectedIndex == 2)
        {
            btnStartDevice3->IsEnabled = enabled;
        }
    }
    else if(name->Equals("StartPreview"))
    {
        if(ScenarioList->SelectedIndex == 0)
        {
            btnStartPreview1->IsEnabled = enabled;
        }
        else if(ScenarioList->SelectedIndex == 1)
        {
            btnStartPreview2->IsEnabled = enabled;
        }
    }
    else if(name->Equals("StartStopRecord"))
    {
        if(ScenarioList->SelectedIndex == 0)
        {
            btnStartStopRecord1->IsEnabled = enabled;
        }
        else if(ScenarioList->SelectedIndex == 1)
        {
            btnStartStopRecord2->IsEnabled = enabled;
        }
        else if(ScenarioList->SelectedIndex == 2)
        {
            btnStartStopRecord3->IsEnabled = enabled;
        }
    }
    else if(name->Equals("TakePhoto"))
    {
        if(ScenarioList->SelectedIndex == 0)
        {
            btnTakePhoto1->IsEnabled = enabled;
        }
        else if(ScenarioList->SelectedIndex == 1)
        {
            btnTakePhoto2->IsEnabled = enabled;
        }
    }
}
