﻿// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved

using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media;


using Windows.Media.Capture;

namespace MediaCaptureCS
{
    partial class MainPage
    {

        const string VIDEO_FILE_NAME = "video.wmv";
        const string AUDIO_FILE_NAME = "audio.m4a";
        const string PHOTO_FILE_NAME = "photo.jpg";

        MediaCapture mediaCaptureMgr;
        Windows.Storage.StorageFile photoStorageFile;
        Windows.Storage.StorageFile recordStorageFile;
        bool recording;
        bool previewing;
        Windows.Devices.Enumeration.DeviceInformationCollection devInfoCollection;

        public MainPage()
        {
            InitializeComponent();
            ScenarioList.SelectionChanged += new SelectionChangedEventHandler(ScenarioList_SelectionChanged);

            // Starting scenario is the first or based upon a previous selection
            ListBoxItem startingScenario = null;
            if (SuspensionManager.SessionState.ContainsKey("SelectedScenario"))
            {
                String selectedScenarioName = SuspensionManager.SessionState["SelectedScenario"] as string;
                startingScenario = this.FindName(selectedScenarioName) as ListBoxItem;
            }
            Application.Current.Suspending += new SuspendingEventHandler(Suspending);
            Application.Current.Resuming +=  new EventHandler<Object>(Resuming);	

            ScenarioList.SelectedItem = startingScenario != null ? startingScenario : Scenario1;

            Scenario1Init();
            Windows.UI.ViewManagement.ApplicationView.GetForCurrentView().ViewStateChanged += new TypedEventHandler<Windows.UI.ViewManagement.ApplicationView, Windows.UI.ViewManagement.ApplicationViewStateChangedEventArgs>(MainPage_ViewStateChanged);
        }

        void DisplayProperties_OrientationChanged(object sender)
        {
            if (DisplayProperties.CurrentOrientation == DisplayOrientations.Portrait || DisplayProperties.CurrentOrientation == DisplayOrientations.PortraitFlipped)
            {
                VisualStateManager.GoToState(this, "Portrait", false);
            }
        }

        void MainPage_ViewStateChanged(Windows.UI.ViewManagement.ApplicationView sender, Windows.UI.ViewManagement.ApplicationViewStateChangedEventArgs args)
        {
            switch (args.ViewState)
            {
                case Windows.UI.ViewManagement.ApplicationViewState.Filled:
                    VisualStateManager.GoToState(this, "Filled", false);
                    break;
                case Windows.UI.ViewManagement.ApplicationViewState.FullScreenLandscape:
                    VisualStateManager.GoToState(this, "FullScreenLandscape", false);
                    break;
                case Windows.UI.ViewManagement.ApplicationViewState.Snapped:
                    VisualStateManager.GoToState(this, "Snapped", false);
                    break;
                case Windows.UI.ViewManagement.ApplicationViewState.FullScreenPortrait:
                    VisualStateManager.GoToState(this, "FullScreenPortrait", false);
                    break;
                default:
                    break;
            }
        }


        void ScenarioList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ResetAll();
            if (ScenarioList.SelectedItem == Scenario1)
            {
                Scenario1Init();
                Scenario1Input.Visibility = Visibility.Visible;
                Scenario1Output.Visibility = Visibility.Visible;
            }
            else if (ScenarioList.SelectedItem == Scenario2)
            {
                Scenario2Init();
                Scenario2Input.Visibility = Visibility.Visible;
                Scenario2Output.Visibility = Visibility.Visible;
            }
            else if (ScenarioList.SelectedItem == Scenario3)
            {
                Scenario3Init();
                Scenario3Input.Visibility = Visibility.Visible;
                Scenario3Output.Visibility = Visibility.Visible;
            }

            if (ScenarioList.SelectedItem != null)
            {
                ListBoxItem selectedListBoxItem = ScenarioList.SelectedItem as ListBoxItem;
                SuspensionManager.SessionState["SelectedScenario"] = selectedListBoxItem.Name;
            }
        }

        public void ResetAll()
        {
            playbackElement1.Source = null;
            previewElement2.Source = null;
            playbackElement2.Source = null;
            imageElement2.Source = null;

            Scenario1Input.Visibility = Visibility.Collapsed;
            Scenario1Output.Visibility = Visibility.Collapsed;
            Scenario1Reset();
            Scenario2Input.Visibility = Visibility.Collapsed;
            Scenario2Output.Visibility = Visibility.Collapsed;
            Scenario2Reset();
            Scenario3Input.Visibility = Visibility.Collapsed;
            Scenario3Output.Visibility = Visibility.Collapsed;
            Scenario3Reset();


        }

        async void Footer_Click(object sender, RoutedEventArgs e)
        {
            await Windows.System.Launcher.LaunchUriAsync(new Uri(((HyperlinkButton)sender).Tag.ToString()));
        }

        void ShowStatusMessage(String text)
        {
            StatusMessage.Text = text;
        }

        void ShowExceptionMessage(Exception ex)
        {
            ShowStatusMessage(ex.Message);
        }


        #region Scenario Specific Code

        void Scenario1Init()
        {
            btnStartDevice1.IsEnabled = true;
            btnStartPreview1.IsEnabled = false;
            btnStartStopRecord1.IsEnabled = false;
            recording = false;
            previewing = false;
            btnStartStopRecord1.Content = "StartRecord";
            btnTakePhoto1.IsEnabled = false;
            previewElement1.Source = null;
            playbackElement1.Source = null;
            imageElement1.Source = null;
            ShowStatusMessage("");
            sldBrightness.IsEnabled = false;
            sldContrast.IsEnabled = false;
            previewCanvas1.Visibility = Visibility.Collapsed;
        }

        void Scenario1Reset()
        {
            previewCanvas1.Visibility = Visibility.Collapsed;
            ShowStatusMessage("");
        }

        void Scenario2Init()
        {
            btnStartDevice2.IsEnabled = true;
            btnStartPreview2.IsEnabled = false;
            btnStartStopRecord2.IsEnabled = false;
            recording = false;
            previewing = false;
            btnStartStopRecord2.Content = "StartRecord";
            btnTakePhoto2.IsEnabled = false;
            previewElement2.Source = null;
            playbackElement2.Source = null;
            imageElement2.Source = null;
            ShowStatusMessage("");
            chkAddRemoveEffect.IsEnabled = false;
            previewCanvas2.Visibility = Visibility.Collapsed;
            EnumerateWebcamsAsync();
        }

        void Scenario2Reset()
        {
            previewCanvas2.Visibility = Visibility.Collapsed;
            ShowStatusMessage("");

        }

        void Scenario3Init()
        {
            btnStartDevice3.IsEnabled = true;
            btnStartStopRecord3.IsEnabled = false;
            recording = false;
            previewing = false;
            playbackElement2.Source = null;
            ShowStatusMessage("");

        }

        void Scenario3Reset()
        {
            ShowStatusMessage("");

        }

        #endregion


        async void Suspending(Object sender, Windows.ApplicationModel.SuspendingEventArgs e)
        {
            try
            {
                if (recording)
                {
                    ShowStatusMessage("Stopping Record on suspension");

                    await mediaCaptureMgr.StopRecordAsync();
                }
                if (previewing)
                {
                    ShowStatusMessage("Stopping Preview on suspension");

                    await mediaCaptureMgr.StopPreviewAsync();

                }
            }
            catch (Exception ex)
            {
                ShowExceptionMessage(ex);
            }
        }

        void Resuming(Object sender, Object e)
        {
            ScenarioList_SelectionChanged(sender, null);
        }

        void lstEnumedDevices_SelectionChanged(Object sender, Windows.UI.Xaml.Controls.SelectionChangedEventArgs e)
        {
            btnStartDevice2.IsEnabled = true;
            btnStartPreview2.IsEnabled = false;
            btnStartStopRecord2.IsEnabled = false;
            recording = false;
            btnStartStopRecord2.Content = "StartRecord";
            btnTakePhoto2.IsEnabled = false;
            previewElement2.Source = null;
            playbackElement2.Source = null;
            imageElement2.Source = null;
            chkAddRemoveEffect.IsEnabled = false;
            chkAddRemoveEffect.IsChecked = false;
            ShowStatusMessage("");
        }


        async void EnumerateWebcamsAsync()
        {
            try
            {
                ShowStatusMessage("Enumerating Webcams...");
                devInfoCollection = null;

                EnumedDeviceList2.Items.Clear();

                devInfoCollection = await Windows.Devices.Enumeration.DeviceInformation.FindAllAsync(Windows.Devices.Enumeration.DeviceClass.VideoCapture);


                if (devInfoCollection == null || devInfoCollection.Count == 0)
                {
                    ShowStatusMessage("No WebCams found.");
                }
                else
                {
                    for (int i = 0; i < devInfoCollection.Count; i++)
                    {
                        var devInfo = devInfoCollection.ElementAt(i);
                        EnumedDeviceList2.Items.Add(devInfo.Name);
                    }
                    EnumedDeviceList2.SelectedIndex = 0;
                    ShowStatusMessage("Enumerating Webcams completed successfully.");
                    btnStartDevice2.IsEnabled = true;
                }

            }
            catch (Exception e)
            {
                ShowExceptionMessage(e);

            }
        }

        async void btnStartDevice_Click(Object sender, RoutedEventArgs e)
        {
            try
            {
                EnableButton(false, "StartDevice");
                ShowStatusMessage("Starting device");
                mediaCaptureMgr = new Windows.Media.Capture.MediaCapture();
                if (ScenarioList.SelectedIndex == 0)
                {
                    await mediaCaptureMgr.InitializeAsync();
                }
                if (ScenarioList.SelectedIndex == 1)
                {
                    var settings = new MediaCaptureInitializationSettings();
                    var chosenDevInfo = devInfoCollection.ElementAt(EnumedDeviceList2.SelectedIndex);
                    settings.VideoDeviceId = chosenDevInfo.Id;
                    await mediaCaptureMgr.InitializeAsync(settings);
                }
                if (ScenarioList.SelectedIndex == 2)
                {
                    var settings = new MediaCaptureInitializationSettings();
                    settings.StreamingCaptureMode = StreamingCaptureMode.Audio;
                    await mediaCaptureMgr.InitializeAsync(settings);
                }

                EnableButton(true, "StartPreview");
                EnableButton(true, "StartStopRecord");
                EnableButton(true, "TakePhoto");

                if (ScenarioList.SelectedIndex == 0)
                {
                    // Brightness
                    SetupVideoDeviceControl(mediaCaptureMgr.VideoDeviceController.Brightness, sldBrightness);
                    // Contrast
                    SetupVideoDeviceControl(mediaCaptureMgr.VideoDeviceController.Contrast, sldContrast);
                }
                if (ScenarioList.SelectedIndex == 1)
                {
                    chkAddRemoveEffect.IsEnabled = true;
                }
                ShowStatusMessage("Device initialized successfully.");
            }
            catch (Exception ex)
            {
                ShowExceptionMessage(ex);
            }
        }


        async void btnStartPreview_Click(Object sender, RoutedEventArgs e)
        {
            previewing = false;
            try
            {
                ShowStatusMessage("Starting preview");
                EnableButton(false, "StartPreview");
                if (ScenarioList.SelectedIndex == 0)
                {
                    previewCanvas1.Visibility = Visibility.Visible;
                    previewElement1.Source = mediaCaptureMgr;
                }
                if (ScenarioList.SelectedIndex == 1)
                {
                    previewCanvas2.Visibility = Visibility.Visible;
                    previewElement2.Source = mediaCaptureMgr;
                }
                await mediaCaptureMgr.StartPreviewAsync();
                previewing = true;
                ShowStatusMessage("Start preview successful");
            }
            catch (Exception ex)
            {
                ShowExceptionMessage(ex);
            }
        }


        async void btnTakePhoto_Click(Object sender, RoutedEventArgs e)
        {
            try
            {
                ShowStatusMessage("Taking photo");
                EnableButton(false, "TakePhoto");

                photoStorageFile = await Windows.Storage.KnownFolders.PicturesLibrary.CreateFileAsync(PHOTO_FILE_NAME);
                ShowStatusMessage("Create photo file successful");
                Windows.Media.MediaProperties.ImageEncodingProperties imageProperties = new Windows.Media.MediaProperties.ImageEncodingProperties();
                imageProperties.Subtype = "JPEG";
                imageProperties.Width = 320;
                imageProperties.Height = 240;

                await mediaCaptureMgr.CapturePhotoToStorageFileAsync(imageProperties, photoStorageFile);

                ShowStatusMessage("Photo taken");

                var stream = await photoStorageFile.OpenAsync(Windows.Storage.FileAccessMode.Read);

                ShowStatusMessage("Photo file opened");

                var bmpimg = new Windows.UI.Xaml.Media.Imaging.BitmapImage();

                bmpimg.SetSource(stream);
                if (ScenarioList.SelectedIndex == 0)
                {
                    imageElement1.Source = bmpimg;
                }
                else if (ScenarioList.SelectedIndex == 1)
                {
                    imageElement2.Source = bmpimg;
                }
                ShowStatusMessage(photoStorageFile.Path);
                EnableButton(true, "TakePhoto");
            }
            catch (Exception ex)
            {
                EnableButton(true, "TakePhoto");
                ShowExceptionMessage(ex);
            }
        }


        async void btnStartStopRecord_Click(Object sender, RoutedEventArgs e)
        {
            try
            {
                String fileName;
                EnableButton(false, "StartStopRecord");

                if (!recording)
                {
                    ShowStatusMessage("Starting Record");

                    if (ScenarioList.SelectedIndex == 0 || ScenarioList.SelectedIndex == 1)
                    {
                        fileName = VIDEO_FILE_NAME;
                    }
                    else
                    {
                        fileName = AUDIO_FILE_NAME;
                    }
                    recordStorageFile = await Windows.Storage.KnownFolders.VideosLibrary.CreateFileAsync(fileName);


                    Windows.Media.MediaProperties.MediaEncodingProfile recordProfile = null;
                    if (ScenarioList.SelectedIndex == 0 || ScenarioList.SelectedIndex == 1)
                    {
                        recordProfile = Windows.Media.MediaProperties.MediaEncodingProfile.CreateMp4(Windows.Media.MediaProperties.VideoEncodingQuality.Auto);
                    }
                    else
                    {
                        recordProfile = Windows.Media.MediaProperties.MediaEncodingProfile.CreateM4a(Windows.Media.MediaProperties.AudioEncodingQuality.Auto);
                    }

                    await mediaCaptureMgr.StartRecordToStorageFileAsync(recordProfile, recordStorageFile);

                    EnableButton(true, "StartStopRecord");
                    recording = true;
                    SwitchRecordButtonContent();
                    ShowStatusMessage("Start Record successful");

                }
                else
                {
                    ShowStatusMessage("Stopping Record");

                    await mediaCaptureMgr.StopRecordAsync();
                    var stream = await recordStorageFile.OpenAsync(Windows.Storage.FileAccessMode.Read);
                    if (ScenarioList.SelectedIndex == 0)
                    {
                        playbackElement1.AutoPlay = true;
                        playbackElement1.SetSource(stream, recordStorageFile.FileType);
                        playbackElement1.Play();
                    }
                    else if (ScenarioList.SelectedIndex == 1)
                    {
                        playbackElement2.AutoPlay = true;
                        playbackElement2.SetSource(stream, recordStorageFile.FileType);
                        playbackElement2.Play();
                    }
                    else if (ScenarioList.SelectedIndex == 2)
                    {
                        playbackElement3.AutoPlay = true;
                        playbackElement3.SetSource(stream, recordStorageFile.FileType);
                        playbackElement3.Play();
                    }
                    EnableButton(true, "StartStopRecord");
                    recording = false;
                    SwitchRecordButtonContent();
                    ShowStatusMessage("Stop Record successful" + recordStorageFile.Path);

                }
            }
            catch (Exception ex)
            {
                EnableButton(true, "StartStopRecord");
                recording = false;
                ShowExceptionMessage(ex);
            }
        }

        async void chkAddRemoveEffect_Checked(Object sender, RoutedEventArgs e)
        {
            try
            {
                chkAddRemoveEffect.IsEnabled = false;
                await mediaCaptureMgr.AddEffectAsync(Windows.Media.Capture.MediaStreamType.VideoPreview, "GrayscaleTransform.GrayscaleEffect", null);
                ShowStatusMessage("Add effect successful");
                chkAddRemoveEffect.IsEnabled = true;
            }
            catch (Exception ex)
            {
                chkAddRemoveEffect.IsEnabled = true;
                ShowExceptionMessage(ex);
            }
        }

        async void chkAddRemoveEffect_Unchecked(Object sender, RoutedEventArgs e)
        {
            try
            {
                chkAddRemoveEffect.IsEnabled = false;
                await mediaCaptureMgr.ClearEffectsAsync(Windows.Media.Capture.MediaStreamType.VideoPreview);
                ShowStatusMessage("Clear effects successful");
                chkAddRemoveEffect.IsEnabled = true;

            }
            catch (Exception ex)
            {
                chkAddRemoveEffect.IsEnabled = true;
                ShowExceptionMessage(ex);
            }
        }



        void SetupVideoDeviceControl(Windows.Media.Devices.MediaDeviceControl videoDeviceControl, Slider slider)
        {
            try
            {
                if ((videoDeviceControl.Capabilities).Supported)
                {
                    slider.IsEnabled = true;
                    slider.Maximum = videoDeviceControl.Capabilities.Max;
                    slider.Minimum = videoDeviceControl.Capabilities.Min;
                    slider.StepFrequency = videoDeviceControl.Capabilities.Step;
                    double controlValue = 0;
                    if (videoDeviceControl.TryGetValue(out controlValue))
                    {
                        slider.Value = controlValue;
                    }
                }
                else
                {
                    slider.IsEnabled = false;
                }
            }
            catch (Exception e)
            {
                ShowExceptionMessage(e);
            }
        }

        // VideoDeviceControllers
        void sldBrightness_ValueChanged(Object sender, Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs e)
        {
            try
            {
                bool succeeded = mediaCaptureMgr.VideoDeviceController.Brightness.TrySetValue(sldBrightness.Value);
                if (!succeeded)
                {
                    ShowStatusMessage("Set Brightness failed");
                }
            }
            catch (Exception ex)
            {
                ShowExceptionMessage(ex);
            }
        }

        void sldContrast_ValueChanged(Object sender, Windows.UI.Xaml.Controls.Primitives.RangeBaseValueChangedEventArgs e)
        {
            try
            {
                bool succeeded = mediaCaptureMgr.VideoDeviceController.Contrast.TrySetValue(sldContrast.Value);
                if (!succeeded)
                {
                    ShowStatusMessage("Set Contrast failed");
                }
            }
            catch (Exception ex)
            {
                ShowExceptionMessage(ex);
            }
        }


        void SwitchRecordButtonContent()
        {
            if (ScenarioList.SelectedIndex == 0)
            {
                if (recording)
                {
                    btnStartStopRecord1.Content = "StopRecord";
                }
                else
                {
                    btnStartStopRecord1.Content = "StartRecord";
                }
            }
            if (ScenarioList.SelectedIndex == 1)
            {
                if (recording)
                {
                    btnStartStopRecord2.Content = "StopRecord";
                }
                else
                {
                    btnStartStopRecord2.Content = "StartRecord";
                }
            }
            if (ScenarioList.SelectedIndex == 2)
            {
                if (recording)
                {
                    btnStartStopRecord3.Content = "StopRecord";
                }
                else
                {
                    btnStartStopRecord3.Content = "StartRecord";
                }
            }
        }
        void EnableButton(bool enabled, String name)
        {
            if (name.Equals("StartDevice"))
            {
                if (ScenarioList.SelectedIndex == 0)
                {
                    btnStartDevice1.IsEnabled = enabled;
                }
                else if (ScenarioList.SelectedIndex == 1)
                {
                    btnStartDevice2.IsEnabled = enabled;
                }
                else if (ScenarioList.SelectedIndex == 2)
                {
                    btnStartDevice3.IsEnabled = enabled;
                }
            }
            else if (name.Equals("StartPreview"))
            {
                if (ScenarioList.SelectedIndex == 0)
                {
                    btnStartPreview1.IsEnabled = enabled;
                }
                else if (ScenarioList.SelectedIndex == 1)
                {
                    btnStartPreview2.IsEnabled = enabled;
                }
            }
            else if (name.Equals("StartStopRecord"))
            {
                if (ScenarioList.SelectedIndex == 0)
                {
                    btnStartStopRecord1.IsEnabled = enabled;
                }
                else if (ScenarioList.SelectedIndex == 1)
                {
                    btnStartStopRecord2.IsEnabled = enabled;
                }
                else if (ScenarioList.SelectedIndex == 2)
                {
                    btnStartStopRecord3.IsEnabled = enabled;
                }
            }
            else if (name.Equals("TakePhoto"))
            {
                if (ScenarioList.SelectedIndex == 0)
                {
                    btnTakePhoto1.IsEnabled = enabled;
                }
                else if (ScenarioList.SelectedIndex == 1)
                {
                    btnTakePhoto2.IsEnabled = enabled;
                }
            }
        }
    };
};
