﻿//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

/// <reference path="base-sdk.js" />

(function () {

    var photoFile = "photo.jpg";
    var deviceList;
    var recordState = false;
    var mediaCaptureMgr,
        captureInitSettings,
        encodingProfile,
        storageFile,
        photoStorage;
    var cameraControlSliders;


    function id(elementId) {
        return document.getElementById(elementId);
    }

    function suspendingHandler(e) {
        if (recordState) {
            stopRecord();
        }
        releaseMediaCapture();
    }

    function resumingHandler(e) {
        scenarioInitialize();
    }

    function errorHandler(e) {      
            sdkSample.displayError(e.message);        
    }

    function enumerateCameras() {
        sdkSample.displayStatus("Enumerating webcams");
        var cameraSelect = id("cameraSelect");
        deviceList = null;
        deviceList = new Array();
        while (cameraSelect.length > 0) {
            cameraSelect.remove(0);
        }
        //Enumerate webcams and add them to the list
        var deviceInfo = Windows.Devices.Enumeration.DeviceInformation;
        deviceInfo.findAllAsync(Windows.Devices.Enumeration.DeviceClass.videoCapture).then(function (devices) {
            // Add the devices to deviceList
            sdkSample.displayStatus("Webcams enumerated");
            if (devices.length > 0) {                
                for (var i = 0; i < devices.length; i++) {
                    deviceList.push(devices[i]);
                    cameraSelect.add(new Option(deviceList[i].name), i);
                }
                //Select the first webcam
                cameraSelect.selectedIndex = 0;
                initCaptureSettings();                

            } else {
                sdkSample.displayError("No camera device is found ");
                // disable buttons.
            }
        }, errorHandler);
    }

    function initCaptureSettings() {
        //Initialize capture settings
        captureInitSettings = null;
        captureInitSettings = new Windows.Media.Capture.MediaCaptureInitializationSettings();
        captureInitSettings.audioDeviceId = "";
        captureInitSettings.videoDeviceId = "";
        captureInitSettings.streamingCaptureMode = Windows.Media.Capture.StreamingCaptureMode.audioAndVideo;
        captureInitSettings.photoCaptureSource = Windows.Media.Capture.PhotoCaptureSource.videoPreview;       
        if (sdkSample.scenarioId === "2") {
            //If the user chosen another webcam, use that webcam by default
            var selectedIndex = id("cameraSelect").selectedIndex;
            captureInitSettings.videoDeviceId = deviceList[selectedIndex].id;
        } else if (sdkSample.scenarioId === "3") {
            //If we are performing audio only capture, set streamingCaptureMode  property to the appropriate value
            captureInitSettings.streamingCaptureMode = Windows.Media.Capture.StreamingCaptureMode.Audio;
        }
    }

    function getCameraSettings() {
        //Set up brightness and contrast controls
        var controller = mediaCaptureMgr.videoDeviceController;
        cameraControlSliders = new Array(
        {
            control: controller.brightness, slider: id("rngBrightness")
        },
        {
            control: controller.contrast, slider: id("rngContrast")
        });
        for (var i = 0; i < cameraControlSliders.length; i++) {
            cameraControlSliders[i].handler = (function (_control, _slider) {
                return function () {
                    _control.trySetValue(parseInt(_slider.value));
                };
            })(cameraControlSliders[i].control, cameraControlSliders[i].slider);
        }
        controller = null;
        window.CollectGarbage();
    }

    function initSlider(controlSlider) {
        var control = controlSlider.control;
        var slider = controlSlider.slider;
        var val = control.tryGetValue();
        slider.disabled = true;
        if (control.capabilities.supported) {
            slider.min = control.capabilities.min;
            slider.max = control.capabilities.max;
            var valObject = control.tryGetValue();
            var v = control.tryGetAuto();
            if (valObject.succeeded) {
                slider.value = valObject.value;
                slider.addEventListener("change", controlSlider.handler, false);
                slider.disabled = false;
            }
        }
        control = null;
        slider = null;
        window.CollectGarbage();
    }

    function releaseMediaCapture() {
        id("previewVideo1").src = null;
        id("previewVideo2").src = null;
        mediaCaptureMgr = null;
        if (cameraControlSliders) {
            for (var i = 0; i < cameraControlSliders.length; i++) {
                cameraControlSliders[i].slider.removeEventListener("change", cameraControlSliders[i].handler, false);
                cameraControlSliders[i].control = null;
                cameraControlSliders[i].handler = null;
            }
        }
        
        window.CollectGarbage(); // make sure camera is released.
    }

    function startDevice() {
        //Initialize media capture with the settings
        sdkSample.displayStatus("Starting device");
        releaseMediaCapture();
        mediaCaptureMgr = new Windows.Media.Capture.MediaCapture();
        mediaCaptureMgr.addEventListener("failed", errorHandler, false);
        mediaCaptureMgr.initializeAsync(captureInitSettings).then(function (result) {           
            id("btnStartDevice" + sdkSample.scenarioId).disabled = true;
            id("btnStartPreview" + sdkSample.scenarioId).disabled = false;
            id("btnRecord" + sdkSample.scenarioId).disabled = false;
            id("btnSnap" + sdkSample.scenarioId).disabled = false;
            getCameraSettings();
            // Initialize sliders.
            if (sdkSample.scenarioId === "1") {
                for (var i = 0; i < cameraControlSliders.length; i++) {
                    cameraControlSliders[i].slider.disabled = false;
                    initSlider(cameraControlSliders[i]);
                }
            }
            sdkSample.displayStatus("Device started");
        },
        errorHandler);
    }

    function startPreview() {
        sdkSample.displayStatus("Starting preview");
        id("btnStartPreview" + sdkSample.scenarioId).disabled = true;
        var video = id("previewVideo" + sdkSample.scenarioId);
        video.src = URL.createObjectURL(mediaCaptureMgr, false);
        video.play();
        sdkSample.displayStatus("Preview started");
    }

    function startAudioDevice() {
        //Initialize audio media capture
        sdkSample.displayStatus("Starting device");
        releaseMediaCapture();
        mediaCaptureMgr = new Windows.Media.Capture.MediaCapture();
        mediaCaptureMgr.addEventListener("failed", errorHandler, false);
        mediaCaptureMgr.initializeAsync(captureInitSettings).then(function (result) {
            id("btnStartDevice" + sdkSample.scenarioId).disabled = true;
            id("btnRecord" + sdkSample.scenarioId).disabled = false;
            sdkSample.displayStatus("Device started");
        },
        errorHandler);
    }

    function startRecord() {
        var btnRecord;
        sdkSample.displayStatus("Starting record ");
        btnRecord = id("btnRecord" + sdkSample.scenarioId);
        btnRecord.disabled = true;
        if (!recordState) {
            // Stop captured video playback.
            var video = id("capturePlayback" + sdkSample.scenarioId);
            if (!video.paused) {
                video.pause();
            }
            // Start recording.
            Windows.Storage.KnownFolders.videosLibrary.createFileAsync("cameraCapture.mp4", Windows.Storage.CreationCollisionOption.generateUniqueName).then(function (newFile) {
                storageFile = newFile;
                encodingProfile = Windows.Media.MediaProperties.MediaEncodingProfile.createMp4(Windows.Media.MediaProperties.VideoEncodingQuality.auto);
                mediaCaptureMgr.startRecordToStorageFileAsync(encodingProfile, storageFile).then(function (result) {
                    recordState = true;
                    btnRecord.innerHTML = "Stop Record";
                    btnRecord.disabled = false;
                    sdkSample.displayStatus("Record Started");
                },
                errorHandler);
            },
            errorHandler);
        }
    }

    function stopRecord() {
        var btnRecord;
        sdkSample.displayStatus("Stopping record");
        btnRecord = id("btnRecord" + sdkSample.scenarioId);
        mediaCaptureMgr.stopRecordAsync().then(function (result) {
            recordState = false;
            btnRecord.innerHTML = "Start Record";
            btnRecord.disabled = false;
            sdkSample.displayStatus("Record Stopped.  File " + storageFile.path + "  ");

            // Playback the recorded video.
            var video = id("capturePlayback" + sdkSample.scenarioId);
            video.src = URL.createObjectURL(storageFile, false);
            video.play();
        },
        errorHandler);        
    }

    function startStopRecord() {
        if (!recordState) {
            startRecord();
        } else {
            stopRecord();
        }
    }


    function startAudioRecord() {
        var btnRecord;
        sdkSample.displayStatus("Starting record ");
        btnRecord = id("btnRecord" + sdkSample.scenarioId);
        btnRecord.disabled = true;
        if (!recordState) {
            // Stop captured video playback.
            var audio = id("capturePlayback" + sdkSample.scenarioId);
            if (!audio.paused) {
                audio.pause();
            }
            // Start recording.
            Windows.Storage.KnownFolders.videosLibrary.createFileAsync("cameraCapture.m4a", Windows.Storage.CreationCollisionOption.generateUniqueName).then(function (newFile) {
                storageFile = newFile;
                encodingProfile = Windows.Media.MediaProperties.MediaEncodingProfile.createM4a(Windows.Media.MediaProperties.AudioEncodingQuality.auto);
                mediaCaptureMgr.startRecordToStorageFileAsync(encodingProfile, storageFile).then(function (result) {
                    recordState = true;
                    btnRecord.innerHTML = "Stop Record";
                    btnRecord.disabled = false;
                    sdkSample.displayStatus("Record Started");
                },
                errorHandler);
            },
            errorHandler);
        }
    }

    function stopAudioRecord() {
        var btnRecord;
        sdkSample.displayStatus("Stopping record");
        btnRecord = id("btnRecord" + sdkSample.scenarioId);
        mediaCaptureMgr.stopRecordAsync().then(function (result) {
            recordState = false;
            btnRecord.innerHTML = "Start Record";
            btnRecord.disabled = false;
            sdkSample.displayStatus("Record Stopped.  File " + storageFile.path + "  ");

            // Playback the recorded video.
            var audio = id("capturePlayback" + sdkSample.scenarioId);
            audio.src = URL.createObjectURL(storageFile, false);
            audio.play();
        },
        errorHandler);        
    }

    function startStopAudioRecord() {
        if (!recordState) {
            startAudioRecord();
        } else {
            stopAudioRecord();
        }
    }
    function addRemoveEffect() {
        //Add gray scale effect
        var selectedEffect = "GrayscaleTransform.GrayscaleEffect";
        if (id("videoEffect").checked) {
            sdkSample.displayStatus("Adding effect");
            mediaCaptureMgr.addEffectAsync(Windows.Media.Capture.MediaStreamType.videoPreview, selectedEffect, null).then(
                    function (result) {
                        sdkSample.displayStatus("Effect added");
                    },
                errorHandler);
        }
        else {
            sdkSample.displayStatus("Clearing effect");
            mediaCaptureMgr.clearEffectsAsync(Windows.Media.Capture.MediaStreamType.videoPreview, null).then(
                    function (result) {
                        sdkSample.displayStatus("Effect cleared");
                    },
                errorHandler);
        }
    }

    function capturePhoto() {
        sdkSample.displayStatus("Taking photo");
        return Windows.Storage.KnownFolders.picturesLibrary.createFileAsync(photoFile).then(
        function (newFile) {
            photoStorage = newFile;
            var photoProperties = new Windows.Media.MediaProperties.ImageEncodingProperties();
            photoProperties.subtype = "JPEG";
            photoProperties.width = 320;
            photoProperties.height = 240;

            mediaCaptureMgr.capturePhotoToStorageFileAsync(photoProperties, photoStorage).then(
            function (result) {
                sdkSample.displayStatus("Photo Taken.  File " + photoStorage.path + "  ");
                var url = URL.createObjectURL(photoStorage, false);
                id("imageTag" + sdkSample.scenarioId).src = url;
            },
            function capturePhotoError(error) {
                sdkSample.displayStatus("An exception occurred trying to capturePhoto: " + error.message);
            });
        },
        function (error) {
            sdkSample.displayStatus("capturePhoto async exception " + error.message);
        });
    }

    function onCameraChange() {
        releaseMediaCapture();
        id("btnStartDevice" + sdkSample.scenarioId).disabled = false;        
        id("btnStartPreview" + sdkSample.scenarioId).disabled = true;        
        id("btnRecord" + sdkSample.scenarioId).disabled = true;        
        id("btnSnap" + sdkSample.scenarioId).disabled = true;
        sdkSample.displayStatus("");
        initCaptureSettings();        
    }

    function initialize() {
        id("scenarios").addEventListener("change", onScenarioChanged, false);
        Windows.UI.WebUI.WebUIApplication.addEventListener("suspending", suspendingHandler, false);
        Windows.UI.WebUI.WebUIApplication.addEventListener("resuming", resumingHandler, false);
        scenarioInitialize();
    }

    function onScenarioChanged() {
        // Clear the status display area
        sdkSample.displayStatus("");
        scenarioInitialize();
    }

    function scenarioInitialize() {
        sdkSample.displayStatus("");
        if (sdkSample.scenarioId === "1") {
            id("btnStartDevice" + sdkSample.scenarioId).disabled = false;
            id("btnStartDevice" + sdkSample.scenarioId).addEventListener("click", startDevice, false);
            id("btnStartPreview" + sdkSample.scenarioId).disabled = true;
            id("btnStartPreview" + sdkSample.scenarioId).addEventListener("click", startPreview, false);
            id("btnRecord" + sdkSample.scenarioId).disabled = true;
            id("btnRecord" + sdkSample.scenarioId).addEventListener("click", startStopRecord, false);
            id("btnSnap" + sdkSample.scenarioId).disabled = true;
            id("btnSnap" + sdkSample.scenarioId).addEventListener("click", capturePhoto, false);
            rngContrast.disabled = true;
            rngBrightness.disabled = true;
            initCaptureSettings();
            
        } else if (sdkSample.scenarioId === "2") {
            id("btnStartDevice" + sdkSample.scenarioId).disabled = false;
            id("btnStartDevice" + sdkSample.scenarioId).addEventListener("click", startDevice, false);
            id("btnStartPreview" + sdkSample.scenarioId).disabled = true;
            id("btnStartPreview" + sdkSample.scenarioId).addEventListener("click", startPreview, false);
            id("btnRecord" + sdkSample.scenarioId).disabled = true;
            id("btnRecord" + sdkSample.scenarioId).addEventListener("click", startStopRecord, false);
            id("btnSnap" + sdkSample.scenarioId).disabled = true;
            id("btnSnap" + sdkSample.scenarioId).addEventListener("click", capturePhoto, false);
            enumerateCameras();
            id("cameraSelect").addEventListener("change", onCameraChange, false);
            id("videoEffect").addEventListener("click", addRemoveEffect, false);
            id("videoEffect").checked = false;
        } else {
            id("btnStartDevice" + sdkSample.scenarioId).disabled = false;
            id("btnRecord" + sdkSample.scenarioId).disabled = true;
            id("btnStartDevice" + sdkSample.scenarioId).addEventListener("click", startAudioDevice, false);
            id("btnRecord" + sdkSample.scenarioId).addEventListener("click", startStopAudioRecord, false);
            initCaptureSettings();            
        }
    }
    document.addEventListener("DOMContentLoaded", initialize, false);
})();
